﻿using System;
using System.Windows;

namespace NonAffineTransformDemo
{
    public struct MatrixNA
    {
        double m11, m12, m13, m21, m22, m23, m31, m32, m33;

        public MatrixNA(double m11, double m12, double m13,
                        double m21, double m22, double m23,
                        double m31, double m32, double m33)
        {
            this.m11 = m11;
            this.m12 = m12;
            this.m13 = m13;
            this.m21 = m21;
            this.m22 = m22;
            this.m23 = m23;
            this.m31 = m31;
            this.m32 = m32;
            this.m33 = m33;
        }

        public double M11 { get { return m11; } }
        public double M12 { get { return m12; } }
        public double M13 { get { return m13; } }
        public double M21 { get { return m21; } }
        public double M22 { get { return m22; } }
        public double M23 { get { return m23; } }
        public double M31 { get { return m31; } }
        public double M32 { get { return m32; } }
        public double M33 { get { return m33; } }

        public static MatrixNA Identity
        {
            get { return new MatrixNA(1, 0, 0, 0, 1, 0, 0, 0, 1); }
        }

        public static MatrixNA operator *(MatrixNA l, MatrixNA r)
        {
            return new MatrixNA(l.M11 * r.M11 + l.M12 * r.M21 + l.M13 * r.M31,
                                l.M11 * r.M12 + l.M12 * r.M22 + l.M13 * r.M32,
                                l.M11 * r.M13 + l.M12 * r.M23 + l.M13 * r.M33,

                                l.M21 * r.M11 + l.M22 * r.M21 + l.M23 * r.M31,
                                l.M21 * r.M12 + l.M22 * r.M22 + l.M23 * r.M32,
                                l.M21 * r.M13 + l.M22 * r.M23 + l.M23 * r.M33,

                                l.M31 * r.M11 + l.M32 * r.M21 + l.M33 * r.M31,
                                l.M31 * r.M12 + l.M32 * r.M22 + l.M33 * r.M32,
                                l.M31 * r.M13 + l.M32 * r.M23 + l.M33 * r.M33);
        }

        public Point Transform(Point pt)
        {
            double den = m13 * pt.X + m23 * pt.Y + m33;

            return new Point((m11 * pt.X + m21 * pt.Y + m31) / den,
                             (m12 * pt.X + m22 * pt.Y + M32) / den);
        }
    }
}
