﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace NonAffineTransformDemo
{
    public partial class NonAffinePolygonTransformer : UserControl
    {
        public static readonly DependencyProperty PolygonsProperty =
            DependencyProperty.Register("Polygons",
                typeof(List<Polygon>),
                typeof(NonAffinePolygonTransformer),
                new PropertyMetadata(null, OnPolygonsChanged));

        public static readonly DependencyProperty MatrixProperty =
            DependencyProperty.Register("Matrix",
                typeof(MatrixNA?),
                typeof(NonAffinePolygonTransformer),
                new PropertyMetadata(null, OnMatrixChanged));

        public NonAffinePolygonTransformer()
        {
            InitializeComponent();
            Polygons = new List<Polygon>();
        }

        public List<Polygon> Polygons
        {
            set { SetValue(PolygonsProperty, value); }
            get { return (List<Polygon>)GetValue(PolygonsProperty); }
        }

        public MatrixNA? Matrix
        {
            set { SetValue(MatrixProperty, value); }
            get { return (MatrixNA?)GetValue(MatrixProperty); }
        }

        static void OnPolygonsChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as NonAffinePolygonTransformer).OnPolygonsChanged(args);
        }

        void OnPolygonsChanged(DependencyPropertyChangedEventArgs args)
        {
            canvas.Children.Clear();

            if (Polygons == null || Polygons.Count == 0 || !Matrix.HasValue)
                return;

            CreatePolygons();
            TransformPoints();
        }

        static void OnMatrixChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            (obj as NonAffinePolygonTransformer).OnMatrixChanged(args);
        }

        void OnMatrixChanged(DependencyPropertyChangedEventArgs args)
        {
            if (Polygons == null || Polygons.Count == 0 || !Matrix.HasValue)
                return;

            if (Polygons.Count != canvas.Children.Count)
                CreatePolygons();

            TransformPoints();
        }

        void CreatePolygons()
        {
            canvas.Children.Clear();

            foreach (Polygon polygonSrc in Polygons)
            {
                Polygon polygonDst = new Polygon();
                polygonDst.Fill = polygonSrc.Fill;
                canvas.Children.Add(polygonDst);
            }
        }

        void TransformPoints()
        {
            for (int i = 0; i < Polygons.Count; i++)
            {
                Polygon polygonSrc = Polygons[i];
                Polygon polygonDst = canvas.Children[i] as Polygon;

                PointCollection pointsDst = polygonDst.Points;
                polygonDst.Points = null;
                pointsDst.Clear();

                foreach (Point pt in polygonSrc.Points)
                    pointsDst.Add(Matrix.Value.Transform(pt));

                polygonDst.Points = pointsDst;
            }
        }
    }
}
