﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace NonAffineTransformDemo
{
    public partial class Page : UserControl
    {
        Dot draggedDot;
        Point startPointMouse;
        Point startPointDot;

        public Page()
        {
            InitializeComponent();
            SetNewTransform();
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs args)
        {
            base.OnMouseLeftButtonDown(args);

            if (args.OriginalSource is Path)
            {
                DependencyObject parent = (args.OriginalSource as Path).Parent;

                if (parent is Dot)
                {
                    draggedDot = parent as Dot;
                    startPointDot = draggedDot.Center;
                    startPointMouse = args.GetPosition(this);
                    CaptureMouse();
                }
            }
        }

        protected override void OnMouseMove(MouseEventArgs args)
        {
            base.OnMouseMove(args);

            if (draggedDot != null)
            {
                Point pointMouse = args.GetPosition(this);

                draggedDot.Center = new Point(startPointDot.X - startPointMouse.X + pointMouse.X,
                                              startPointDot.Y - startPointMouse.Y + pointMouse.Y);
                SetNewTransform();
            }
        }

        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs args)
        {
            base.OnMouseLeftButtonUp(args);

            if (draggedDot != null)
            {
                ReleaseMouseCapture();
                draggedDot = null;
            }
        }

        void SetNewTransform()
        {
            // Scale transform
            MatrixNA S = new MatrixNA(1.0 / 192, 0, 0, 0, 1.0 / 192, 0, 0, 0, 1);

            // See http://www.charlespetzold.com/blog/2007/08/250638.html for math that follows

            // Affine transform
            double offsetX = dotUL.Center.X;
            double offsetY = dotUL.Center.Y;
            double m11 = (dotUR.Center.X - offsetX) / 1;
            double m12 = (dotUR.Center.Y - offsetY) / 1;
            double m21 = (dotLL.Center.X - offsetX) / 1;
            double m22 = (dotLL.Center.Y - offsetY) / 1;

            MatrixNA A = new MatrixNA(m11, m12, 0, m21, m22, 0, offsetX, offsetY, 1);

            // Non-Affine transform
            double den = m11 * m22 - m12 * m21;
            double a = (m22 * dotLR.Center.X - m21 * dotLR.Center.Y + m21 * offsetY - m22 * offsetX) / den;
            double b = (m11 * dotLR.Center.Y - m12 * dotLR.Center.X + m12 * offsetX - m11 * offsetY) / den;

            m11 = a / (a + b - 1);
            m22 = b / (a + b - 1);
            double m13 = m11 - 1;
            double m23 = m22 - 1;

            MatrixNA B = new MatrixNA(m11, 0, m13, 0, m22, m23, 0, 0, 1);

            // Product of three transforms
            MatrixNA M = S * B * A;

            xformer.Matrix = M;

            textM11.Text = M.M11.ToString("F3");
            textM12.Text = M.M12.ToString("F3");
            textM13.Text = M.M13.ToString("F3");
            textM21.Text = M.M21.ToString("F3");
            textM22.Text = M.M22.ToString("F3");
            textM23.Text = M.M23.ToString("F3");
            textM31.Text = M.M31.ToString("F3");
            textM32.Text = M.M32.ToString("F3");
        }
    }
}
